@@ -59,8 +59,6 @@ static void decodeDWS(char *s, levelTab lt);
 static void decodeCIR(char *s, levelTab lt);
 
 /* IMPORTANT NOTE: the number of cartridges is to be confirmed for each type */
-/* Must modify MAX_NUMBER_OF_MODELS each time you touch to existingPrinters */
-#define MAX_NUMBER_OF_MODELS 21
 
 struct
 { char *model; /* The model identifier */
@@ -69,20 +67,28 @@ struct
     unsigned char numberOfColors;
     char cartridgeTypes[10];
   } chdTypes[4];
-} existingPrinters [MAX_NUMBER_OF_MODELS] =
+} existingPrinters [] =
   {
+    {"iP1800",
+     {{"BK,CL",2,{CARTRIDGE_BLACK,CARTRIDGE_COLOR}},
+      {"BK",1,{CARTRIDGE_BLACK}},
+      {NULL,RESPONSE_INVALID,{}}}},
     {"iP1000",
      {{"BK",1,{CARTRIDGE_BLACK}},
       {"CL",2,{CARTRIDGE_BLACK,CARTRIDGE_COLOR}},
       {NULL,RESPONSE_INVALID,{}}}},
-    {"ip2200",
-     {{"BK",1,{CARTRIDGE_BLACK}},
-      {"CL",2,{CARTRIDGE_BLACK,CARTRIDGE_COLOR}},
+    {"iP2200",
+     {{"BK,CL",2,{CARTRIDGE_BLACK,CARTRIDGE_COLOR}},
+      {"BK",1,{CARTRIDGE_BLACK}},
       {NULL,RESPONSE_INVALID,{}}}},
     {"S300",
      {{"VC",2,{CARTRIDGE_BLACK,CARTRIDGE_COLOR}},
       {"CL",2,{CARTRIDGE_BLACK,CARTRIDGE_COLOR}},
       {NULL,RESPONSE_INVALID,{}}}},
+    {"S520",
+     {{"CL",4,{CARTRIDGE_BLACK,CARTRIDGE_CYAN,
+               CARTRIDGE_MAGENTA,CARTRIDGE_YELLOW}},
+      {NULL,RESPONSE_INVALID,{}}}},
     {"i550",
      {{"VC",4,{CARTRIDGE_BLACK,CARTRIDGE_CYAN,
                CARTRIDGE_MAGENTA,CARTRIDGE_YELLOW}},
@@ -117,6 +123,21 @@ struct
                CARTRIDGE_YELLOW,CARTRIDGE_MAGENTA,
                CARTRIDGE_CYAN}},
       {NULL,RESPONSE_INVALID,{}}}},
+    {"i950",
+     {{"LS",6,{CARTRIDGE_CYAN,CARTRIDGE_LIGHTCYAN,
+               CARTRIDGE_BLACK,CARTRIDGE_YELLOW,
+               CARTRIDGE_MAGENTA,CARTRIDGE_LIGHTMAGENTA}},
+      {NULL,RESPONSE_INVALID,{}}}},
+    {"i965",
+     {{"LS",6,{CARTRIDGE_BLACK,CARTRIDGE_CYAN,
+               CARTRIDGE_MAGENTA,CARTRIDGE_YELLOW,
+               CARTRIDGE_PHOTOCYAN,CARTRIDGE_PHOTOMAGENTA}},
+      {NULL,RESPONSE_INVALID,{}}}},
+    {"i960",
+     {{"LS",6,{CARTRIDGE_BLACK,CARTRIDGE_CYAN,
+               CARTRIDGE_MAGENTA,CARTRIDGE_YELLOW,
+               CARTRIDGE_PHOTOCYAN,CARTRIDGE_PHOTOMAGENTA}},
+      {NULL,RESPONSE_INVALID,{}}}},
     {"i990",
      {{"LS",6,{CARTRIDGE_BLACK,CARTRIDGE_CYAN,
                CARTRIDGE_MAGENTA,CARTRIDGE_YELLOW,
@@ -139,6 +160,18 @@ struct
       {"CL",5,{CARTRIDGE_BLACK,CARTRIDGE_PHOTOBLACK,
                CARTRIDGE_CYAN,CARTRIDGE_MAGENTA,CARTRIDGE_YELLOW,}},
       {NULL,RESPONSE_INVALID,{}}}},
+    {"iP4300",
+     {{"VC",5,{CARTRIDGE_BLACK,CARTRIDGE_PHOTOBLACK, 
+               CARTRIDGE_CYAN,CARTRIDGE_MAGENTA,CARTRIDGE_YELLOW,}},
+      {"CL",5,{CARTRIDGE_BLACK,CARTRIDGE_PHOTOBLACK,
+               CARTRIDGE_CYAN,CARTRIDGE_MAGENTA,CARTRIDGE_YELLOW,}},
+      {NULL,RESPONSE_INVALID,{}}}},
+    {"iP4500",
+     {{"VC",5,{CARTRIDGE_BLACK,CARTRIDGE_PHOTOBLACK, 
+               CARTRIDGE_CYAN,CARTRIDGE_MAGENTA,CARTRIDGE_YELLOW,}},
+      {"CL",5,{CARTRIDGE_BLACK,CARTRIDGE_PHOTOBLACK,
+               CARTRIDGE_CYAN,CARTRIDGE_MAGENTA,CARTRIDGE_YELLOW,}},
+      {NULL,RESPONSE_INVALID,{}}}},
      {"iP3000",
      {{"VC",4,{CARTRIDGE_BLACK,CARTRIDGE_CYAN,
                CARTRIDGE_MAGENTA,CARTRIDGE_YELLOW}},
@@ -165,6 +198,10 @@ struct
      {{"CL",5,{CARTRIDGE_BLACK,CARTRIDGE_PHOTOBLACK,
                CARTRIDGE_CYAN,CARTRIDGE_MAGENTA,CARTRIDGE_YELLOW}},
       {NULL,RESPONSE_INVALID,{}}}},
+    {"iP5200",
+     {{"CL",5,{CARTRIDGE_BLACK,CARTRIDGE_PHOTOBLACK,
+               CARTRIDGE_CYAN,CARTRIDGE_MAGENTA,CARTRIDGE_YELLOW}},
+      {NULL,RESPONSE_INVALID,{}}}},
     {"MP160",
      {{"BK,CL",2,{CARTRIDGE_BLACK,CARTRIDGE_COLOR}},
       {NULL,RESPONSE_INVALID,{}}}},
@@ -187,6 +224,9 @@ struct
       {NULL,RESPONSE_INVALID,{}}}}
 };
 
+const static int MAX_NUMBER_OF_MODELS = 
+  sizeof(existingPrinters) / sizeof(existingPrinters[0]);
+
 /* Some Canon printers give a binary ink level indicator :
   LOW (attributed to 20% remaining) or not LOW, 100% remaining.
   Some other printers give more precise indicator, 40 or 70% remaining. */
@@ -214,7 +254,7 @@ int get_ink_level_canon(const int port, const char* device_file,
   char cmdGetColors[] = 
     "SSR=BST,SFA,CHD,CIL,CIR,HRI,DBS,DWS,DOC,DSC,DJS,CTK,HCF;";
   char command[256];
-  unsigned char length;
+  int length;
   int i = 0;
   char buffer[BUFLEN];
   char *indexDOC = NULL, *indexDWS = NULL, *indexCHD = NULL, *indexCIR = NULL;
@@ -487,7 +527,8 @@ static void decodeCHD(char *s, struct ink_level *level) {
 }
 
 static void decodeCIR(char *s, levelTab lt) {
-  char *keys[9]={",K=",",BK=",",PBK=",",LC=",",LM=",",Y=",",M=",",C=",",CL="}, *z;
+  char *keys[9]={",K=",",BK=",",PBK=",",LC=",",LM=",",Y=",",M=",",C=",",CL="};
+  char *z;
   int values[9]={0,0,0,0,0,0,0,0,0}, i;
 
   s+=3;
